﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Hardware.SJJ;
using System.Threading;

namespace SPI_Serial
{
    public class Program
    {
        public static void Main()
        {
            Debug.Print(Resources.GetString(Resources.StringResources.String1));
            App myApp = new App();
            myApp.Run();
        }

        public class App
        {
            
//->            static SPI.Configuration myMaximSPIPortConfig = new SPI.Configuration(Cpu.Pin.GPIO_NONE, false, 0, 0, false, true, 50, SPI.SPI_module.SPI1);
            static SPI.Configuration myMaximSPIPortConfig = new SPI.Configuration(Cpu.Pin.GPIO_NONE, false, 0, 0, false, true, 5000, SPI.SPI_module.SPI1);
            SPI mySerialSPIPort = new SPI(myMaximSPIPortConfig);

//->            const UInt16 uiMaximConfig = 0xC00C;   // 4800, 8, N, 1; no interrupts
            const UInt16 uiMaximConfig = 0xC00B;   // 9600, 8, N, 1; no interrupts
//->            const UInt16 uiMaximConfig = 0xC001;   // 115,200, 8, N, 1; no interrupts
            const UInt16 uiMaximConfigRM = 0x0400;  // Enable receive interrupt
//->            const UInt16 uiMaximConfigRM = 0x2400;  // Disable FIFO & Enable receive interrupt
            const UInt16 uiMaximWritePrefix = 0x8000;   // OR with 8-bit data on lower byte
            const UInt16 uiMaximReadCommand = 0x0000;
            const UInt16 uiMaximReadDataValid = 0x8000;   // AND with DOUT and 
            //> 0 if data valid
            const UInt16 uiMaximTxFull = 0x4000;   // AND with DOUT and if 0 Tx full
            const UInt16 uiMaximReadDataMask = 0x00FF;   // AND with data read for data out

            ushort[] uiReadCommand = new ushort[] { uiMaximReadCommand };
            ushort[] uiReadData = new ushort[1];
            ushort[] uiWriteData = new ushort[1];
            ushort[] uiWriteReadData = new ushort[1];

            InterruptPort EGPIO2;
            OutputPort myGreenLED;

            // Set up a line buffer
            const int c_iBuffSize = 4096;
            ushort[] uiLineBuffer = new ushort[c_iBuffSize];    //circular buffer
            char[] cOutString = new char[c_iBuffSize];
            char[] cOutString2 = new char[c_iBuffSize];
            static int iWriteIndex = 0;
            static int iReadIndex = 0;



            public void Run()
            {


                EGPIO2 = new InterruptPort(Pins.EGPIO2_HDR3_15, false, Port.ResistorMode.Disabled, InterruptModes.InterruptEdgeLow);
                EGPIO2.OnInterrupt += new NativeEventHandler(EGPIO2_OnInterrupt);

                myGreenLED = new OutputPort(Pins.GREEN_LED, false);

                // Write serial port configuration 
                uiWriteData[0] = uiMaximConfig;
                uiWriteData[0] |= uiMaximConfigRM;  //Enable receive interrupt
                mySerialSPIPort.WriteRead(uiWriteData, uiReadData);

                // Character echo loop & status LED flash loop
                uint iLoopCount = 0;
                uint iValidReceiveCount = 0;
                bool bLEDState = true;
                while (true)
                {
                    // Check read buffer and echo back the characters
                    if (iWriteIndex != iReadIndex)
                    {
                        uiWriteData[0] = uiLineBuffer[iWriteIndex];
                        uiWriteData[0] &= uiMaximReadDataMask;
                        uiWriteData[0] |= uiMaximWritePrefix;
                        mySerialSPIPort.WriteRead(uiWriteData, uiWriteReadData);

                        if ((uiWriteReadData[0] & uiMaximTxFull) != 0)
                        {
                            // If write buffer not full, increment the index otherwise send it again next time around
                            iWriteIndex++;
                            if (iWriteIndex >= c_iBuffSize)
                            {
                                // Wrap index
                                iWriteIndex = 0;
                            }
                        }


                        if ((uiMaximReadDataValid & uiWriteReadData[0]) != 0)
                        {
                            // Figure out if I want to store and valid character that I get back
                            iValidReceiveCount++;
                        }
                    }

                    //Idle loop; flash green LED to show activity without using a Thread.Sleep
                    if ((iLoopCount++ % 1000) == 0)
                    {
                        if (bLEDState)
                        {
                            bLEDState = false;
                        }
                        else
                        {
                            bLEDState = true;

                            //Output the valid receive char count on WriteRead
                            if (iValidReceiveCount > 0)
                            {
                                Debug.Print(iValidReceiveCount.ToString());

                                // Reset count
                                iValidReceiveCount = 0;
                            }
                        }
                        myGreenLED.Write(bLEDState);
                    }
                }



            }

            void EGPIO2_OnInterrupt(uint data1, uint data2, DateTime time)
            {
                // Do NULL reads only in this interrupt handler
                mySerialSPIPort.WriteRead(uiReadCommand, uiReadData);

                //If data valid, store it in the circular buffer
                if ((uiMaximReadDataValid & uiReadData[0]) != 0)
                {
                    do
                    {
                        uiLineBuffer[iReadIndex++] = uiReadData[0];

                        // Check for read index wrap
                        if (iReadIndex >= c_iBuffSize)
                        {
                            // Wrap buffer
                            iReadIndex = 0;
                        }

                        mySerialSPIPort.WriteRead(uiReadCommand, uiReadData);
                    }
                    while ((uiMaximReadDataValid & uiReadData[0]) != 0);
                }
            }
        }
    }
}
